#### 309. 买卖股票的最佳时机含冷冻期

难度：中等

---

给定一个整数数组`prices`，其中第  `prices[i]` 表示第 `_i_` 天的股票价格 。​

设计一个算法计算出最大利润。在满足以下约束条件下，你可以尽可能地完成更多的交易（多次买卖一支股票）:

*   卖出股票后，你无法在第二天买入股票 (即冷冻期为 1 天)。

 **注意：** 你不能同时参与多笔交易（你必须在再次购买前出售掉之前的股票）。

 **示例 1:** 

```
输入: prices = [1,2,3,0,2]
输出: 3 
解释: 对应的交易状态为: [买入, 卖出, 冷冻期, 买入, 卖出]
```

 **示例 2:** 

```
输入: prices = [1]
输出: 0
```

 **提示：** 

*   `1 <= prices.length <= 5000`
*   `0 <= prices[i] <= 1000`

---

多维动态规划：

一共有三种状态：

- `dp[i][0]`: 不持股且当天不卖出，若要达到这种状态，则有两种情况，前一天的股票**不卖出或者卖出**，即 `dp[i][0] = Math.max(dp[i - 1][0], dp[i - 1][1])`。
- `dp[i][1]`: 不持股且当天卖出，若要达到这种状态，只有一种情况，就是前一天股票持有的情况下的最大收益加上今天股票的价格，即 `dp[i][1] = dp[i - 1][2] + prices[i]`。
- `dp[i][2]`: 持股，若要达到这种状态，有可能是**前一天持股的最大收益或者今天买入**，即 `dp[i][2] = Math.max(dp[i - 1][2], dp[i - 1][0] - prices[i])`。

```Java
// 思路：
//     考虑有多少种状态，每种状态有哪些选择，或者是做了哪些选择后得到哪种状态。
//     注意：到底是先选择了才有状态，还是先由状态才能选择。这里是先选择了，才有状态

// 状态类型有2种：天数和是否持有。
//     天数：一共为1-n天
//     是否持有：分为持有状态、没持有状态1、没持有状态2。
//         持有状态：选择 无处理 和 买入 都有可能达到该状态
//         没持有状态1：选择 无处理 后达到该状态。
//         没持有状态2：选择 卖出 后达到该状态。注意，卖出后进入一天的冻结期。
//     注意：这里为什么要分两种没持有状态，这是为了便于后续状态转移，如果不区分这两种状态，状态转移没法确定当天是否可以进行买入操作。

// dp表示的含义：
//     dp[i][2] : 第i天为没持有状态2时，此时的最大利润
//     dp[i][1] : 第i天为没持有状态1时，此时的最大利润
//     dp[i][0] ： 第i天为持有状态时，此时的最大利润
// 状态转移方程：
//     dp[i][0]: 第i天为持有状态时，此时的最大利润
//         无处理后达到该状态： dp[i][0] = dp[i-1][0] // 第i天没有处理就持有股票，证明上一天也持有
//         买入后达到该状态： dp[i][0] = dp[i-1][1]-prices[n] // 第i天能买入股票，证明上一天没持有股票，且没进行卖出操作
//         所以dp[i][0] = max(dp[i-1][0], dp[i-1][1]-prices[n]); // 这里思考个问题，两种情况都能到达这个状态的话，那如何选择？为什么是取他们的max？
//     dp[i][1]: 第i天为没持有状态1时，此时的最大利润
//         无处理后达到该状态： dp[i][1] = max(dp[i-1][1], dp[i-1][2]) // 有两种到达该状态的情况，取最大那个
//     dp[i][2]: 第i天为没持有状态2时，此时的最大利润
//         卖出后达到该状态： dp[i][2] = dp[i-1][0]+prices[i]

// 最后max(dp[n-1][1], dp[n-1][2])就是题目所需答案。即第n-1天没持有股票时的最大收益
//=======================================================================================
class Solution {
    public int maxProfit(int[] prices) {
        int n = prices.length;
        int[][] dp = new int[n][3];
        dp[0][2] = -1 * prices[0];  // dp[i][0]: 不持股且当天不卖出; dp[i][1]: 不持股且当天卖出; dp[i][2]: 持股
        for(int i = 1; i < n; i++){
            dp[i][0] = Math.max(dp[i - 1][0], dp[i - 1][1]);
            dp[i][1] = dp[i - 1][2] + prices[i];
            dp[i][2] = Math.max(dp[i - 1][2], dp[i - 1][0] - prices[i]);
        }
        return Math.max(dp[n - 1][0], dp[n - 1][1]);
    }
}
```