#### 215. 数组中的第K个最大元素

难度：中等

---

给定整数数组 `nums` 和整数 `k`，请返回数组中第 ` **k** ` 个最大的元素。

请注意，你需要找的是数组排序后的第 `k` 个最大的元素，而不是第 `k` 个不同的元素。

你必须设计并实现时间复杂度为 `O(n)` 的算法解决此问题。

 **示例 1:** 

```
输入: [3,2,1,5,6,4], k = 2
输出: 5
```

 **示例 2:** 

```
输入: [3,2,3,1,2,4,5,5,6], k = 4
输出: 4
```

 **提示：** 

*   `1 <= k <= nums.length <= 10^5`
*   `-10^4 <= nums[i] <= 10^4`

---

方法一，优先队列：

求第 K 个**最大/小元素**，换言之，**求前 K 个最大/小元素的最小/大值**。也就是构造容量为 K 的**小/大顶堆**。时间复杂度：O(NlogK)。

- 当堆的容量小于 K 时，将元素直接放入堆中。

- 当堆的容量等于 K 时，并且元素大/小于小/大顶堆的堆顶元素时，弹出堆顶元素，插入该元素。

为什么不使用最大堆？

- 因为使用最小堆的空间复杂度只有 O(K)，而使用最大堆的话空间复杂度将是 O(N)!

```java
class Solution {
    public int findKthLargest(int[] nums, int k) {
        PriorityQueue<Integer> minheap = new PriorityQueue<>();
        for(int i = 0; i < nums.length; i++){
            if(minheap.size() < k) minheap.offer(nums[i]);
            else if(nums[i] > minheap.peek()){
                minheap.poll();
                minheap.offer(nums[i]);
            }
        }
        return minheap.poll();
    }
}
```

方法二，[快速排序](../Sort/912.%20%E5%BF%AB%E9%80%9F%E6%8E%92%E5%BA%8F.md):

考虑到**快速排序每次循环都会固定一个下标的元素**，因此使用类似快速排序的方式且并不需要将整个数组排序也可以找到第 K 个最大元素。时间复杂度：O(N)。

```java
class Solution {
    public int findKthLargest(int[] nums, int k) {
        int n = nums.length, target = n - k, left = 0, right = n - 1;
        while(true){
            int pivot = findPivot(nums, left, right);
            if(pivot == target){
                return nums[target];
            }
            else if(pivot < target) left = pivot + 1;
            else right = pivot - 1;
        }   
    }

    public int findPivot(int[] nums, int left, int right){
        int index = new Random().nextInt(right - left + 1) + left;
        swap(nums, index, left);
        int lt = left, i = left + 1, gt = right + 1;
        while(i < gt){
            if(nums[i] < nums[left]){
                swap(nums, i++, ++lt);
            }
            else if(nums[i] > nums[left]){
                swap(nums, i, --gt);
            }
            else i++;
        }
        swap(nums, left, lt);
        return lt;
    }

    public void swap(int[] nums, int index1, int index2){
        int temp = nums[index1];
        nums[index1] = nums[index2];
        nums[index2] = temp;
    }
}
```